@@ -4,7 +4,7 @@ require Exporter;
 use strict;
 use vars qw(@ISA @EXPORT);
 @ISA = qw(Exporter);
-@EXPORT = qw(do_backup do_restore ok_dirs_match);
+@EXPORT = qw(do_backup do_restore ok_dirs_match ok_files_match);
 
 use Test::More;
 use FindBin qw($Bin);
@@ -27,8 +27,9 @@ END {
 
 sub do_backup {
     my %opts = @_;
-    my $with_confsec = delete $opts{'with_confsec'} || sub {};
-    my $with_root    = delete $opts{'with_root'}    || sub {};
+    my $with_confsec    = delete $opts{'with_confsec'} || sub {};
+    my $with_targetsec  = delete $opts{'with_targetsec'} || sub {};
+    my $with_root       = delete $opts{'with_root'}    || sub {};
     die if %opts;
 
     my $initer = shift;
@@ -53,13 +54,14 @@ sub do_backup {
 
 
     $confsec = Brackup::ConfigSection->new("TARGET:test_restore");
-    $confsec->add("type" => "Filesystem");
+    $with_targetsec->($confsec);
+    $confsec->add("type" => "Filesystem") unless exists $confsec->{type};
     $confsec->add("inventory_db" => $inv_filename);
     $confsec->add("path" => $backup_dir);
     $conf->add_section($confsec);
 
     my $target = $conf->load_target("test_restore");
-    ok($target, "have a target");
+    ok($target, "have a target ($target)");
 
     my $backup = Brackup::Backup->new(
                                       root    => $root,
@@ -81,19 +83,30 @@ sub do_backup {
 }
 
 sub do_restore {
-    my $backup_file = shift;
+    my ($backup_file, %opts) = @_;
+    my $prefix     = delete $opts{'prefix'} || "";   # default is restore everything
+    my $restore_should_die = delete $opts{'restore_should_die'};
+    die if %opts;
     my $restore_dir = tempdir( CLEANUP => 1 );
     ok_dir_empty($restore_dir);
 
     my $restore = Brackup::Restore->new(
                                         to     => $restore_dir,
-                                        prefix => "",  # backup everything
+                                        prefix => $prefix,
                                         file   => $backup_file,
                                         );
     ok($restore, "have restore object");
-    ok(eval { $restore->restore; }, "did the restore")
-        or die "restore failed: $@";
-    return $restore_dir;
+    my $rv = eval { $restore->restore; };
+    if ($restore_should_die) {
+        ok(! defined $rv, "restore died: $@") 
+            or die "restore unexpectedly succeeded";
+        return;
+    }
+    else {
+        ok($rv, "did the restore") 
+            or die "restore failed: $@";
+        return $restore_dir;
+    }
 }
 
 sub ok_dirs_match {
@@ -113,6 +126,23 @@ sub ok_dirs_match {
     }
 }
 
+sub ok_files_match {
+    my ($after, $before) = @_;
+
+    my $pre_ls  = file_meta($before);
+    my $post_ls = file_meta($after);
+
+    if ($has_diff) {
+        use Data::Dumper;
+        my $pre_dump = Dumper($pre_ls);
+        my $post_dump = Dumper($post_ls);
+        my $diff = Text::Diff::diff(\$pre_dump, \$post_dump);
+        is($diff, "", "files match");
+    } else {
+        is_deeply($post_ls, $pre_ls, "files match");
+    }
+}
+
 sub ok_dir_empty {
     my $dir = shift;
     unless (-d $dir) { ok(0, "not a dir"); return; }
@@ -120,6 +150,33 @@ sub ok_dir_empty {
     is_deeply([ sort readdir($dh) ], ['.', '..'], "dir is empty: $dir");
 }
 
+sub file_meta {
+    my $path = shift;
+    my $st = File::stat::lstat($path);
+
+    my $meta = {};
+    $meta->{size} = $st->size unless -d $path;
+    $meta->{is_file} = 1 if -f $path;
+    $meta->{is_link} = 1 if -l $path;
+    if ($meta->{is_link}) {
+        $meta->{link} = readlink $path;
+    } else {
+        # we ignore these for links, since Linux doesn't let us restore anyway,
+        # as Linux as no lutimes(2) syscall, as of Linux 2.6.16 at least
+        $meta->{atime} = $st->atime if 0; # TODO: make tests work with atimes
+        $meta->{mtime} = $st->mtime;
+        $meta->{mode}  = sprintf('%#o', $st->mode & 0777);
+    }
+
+    # the gpg tests open/close the rings in the root, so
+    # mtimes get bumped around or something.  the proper fix
+    # is too ugly for what it's worth, so let's just ignore
+    # the mtime of top-level
+    delete $meta->{mtime} if $path eq ".";
+
+    return $meta;
+}
+
 # given a directory, returns a hashref of its contentn
 sub dir_structure {
     my $dir = shift;
@@ -132,29 +189,7 @@ sub dir_structure {
         preprocess => sub { return sort @_ },
         wanted => sub {
             my $path = $_;
-            my $st = File::stat::lstat($path);
-
-            my $meta = {};
-            $meta->{size} = $st->size unless -d $path;
-            $meta->{is_file} = 1 if -f $path;
-            $meta->{is_link} = 1 if -l $path;
-            if ($meta->{is_link}) {
-                $meta->{link} = readlink $path;
-            } else {
-                # we ignore these for links, since Linux doesn't let us restore anyway,
-                # as Linux as no lutimes(2) syscall, as of Linux 2.6.16 at least
-                $meta->{atime} = $st->atime if 0; # TODO: make tests work with atimes
-                $meta->{mtime} = $st->mtime;
-                $meta->{mode}  = sprintf('%#o', $st->mode & 0777);
-            }
-
-            # the gpg tests open/close the rings in the root, so
-            # mtimes get bumped around or something.  the proper fix
-            # is too ugly for what it's worth, so let's just ignore
-            # the mtime of top-level
-            delete $meta->{mtime} if $path eq ".";
-
-            $files{$path} = $meta;
+            $files{$path} = file_meta($path);
         },
     }, ".");
 
@@ -164,3 +199,5 @@ sub dir_structure {
 
 
 1;
+
+# vim:et:sw=4
